using System;

namespace com.ibm.gre
{
	/// <summary>
	/// A copy of the Records Manager API
	/// com.ibm.gre.engine.util.AttributeControlMask class.
	/// Bit mask values used in Component Attribute Definitions.
	/// These values refer specifically to the attribute in the Field Element named "AttrCntrlMsk".
	/// They are used to specify certain constraints that have been imposed
	/// on the the Field (or Component Attribute).
	/// Names in this class that start with SYSTEM_ indicate that the bit value can
	/// only be set by the System. 
	/// </summary>
	public class IRMAttrCntrlMsk
	{
		public IRMAttrCntrlMsk(){}

		/// <summary>
		/// Indicates that their are no constraints on the Attribute.
		/// </summary>
		public static int FULL_CONTROL = 0;

		/// <summary>
		/// When set Indicates that the Attribute is Mandatory.
		/// </summary>
		public static int USER_FIELD_IS_MANDATORY = 1;

		/// <summary>
		/// When set Indicates that values for this Attribute must come only from the PickList
		/// that is associated with this Attribute.
		/// </summary>
		public static int USER_LIMIT_TO_LIST = 2;

		/// <summary>
		/// When set Indicates that the value for this Attribute can only be set by the System.
		/// </summary>
		public static int SYSTEM_SET_BY_SYSTEM = 4;

		/// <summary>
		/// When set Indicates that Mandatory bit cannot be changed.
		/// </summary>
		public static int SYSTEM_FREEZE_MANDATORY_FLAG = 8;

		/// <summary>
		/// When set Indicates that this Attribute cannot be Fetched.
		/// </summary>
		public static int SYSTEM_DENY_FETCH = 16;

		/// <summary>
		/// When set Indicates that this Attribute cannot be Queryed.
		/// </summary>
		public static int SYSTEM_DENY_QUERY = 32;
	
		/// <summary>
		/// When set indicates the field level permissions of the field cannot be changed.
		/// </summary>
		public static int SYSTEM_FREEZE_FIELD_ACL = 64;
	
		/// <summary>
		/// When set indicates that the value for the field is an XML formatted string
		/// representing a System object.
		/// </summary>
		public static int SYSTEM_VIRTUAL_FIELD = 128;		
	
	
		public static bool isMandatory(int mask)
		{
			return (((mask & USER_FIELD_IS_MANDATORY) != 0) ? true : false);	
		}
	
		public static bool isSystemFreezeMandatory(int mask)
		{
			return (((mask & SYSTEM_FREEZE_MANDATORY_FLAG) != 0) ? true : false);	
		}
	
		public static bool isReadOnly(int mask)
		{
			return (((mask & SYSTEM_SET_BY_SYSTEM) != 0) ? true : false);	
		}
	
		public static bool isViewable(int mask)
		{		
			return (((mask & SYSTEM_DENY_FETCH) != SYSTEM_DENY_FETCH) ? true : false);
		}	
	
		public static bool isACLSystemFreeze(int mask)
		{
			return (((mask & SYSTEM_FREEZE_FIELD_ACL) == SYSTEM_FREEZE_FIELD_ACL) ? true : false);
		}
	
		public static bool isSetBySystem(int mask)
		{
			return (((mask & SYSTEM_SET_BY_SYSTEM) == SYSTEM_SET_BY_SYSTEM) ? true : false);
		}
	
		public static bool isQueryable(int mask)
		{
			return (((mask & SYSTEM_DENY_QUERY) != SYSTEM_DENY_QUERY) ? true : false);
		}
	
		public static bool isVirtual(int mask)
		{
			return (((mask & SYSTEM_VIRTUAL_FIELD) == SYSTEM_VIRTUAL_FIELD) ? true : false);
		}
		public static bool isLimitToList(int mask)
		{
			return (((mask & USER_LIMIT_TO_LIST) == USER_LIMIT_TO_LIST) ? true : false);
		}
	}
}
