/*
 * IBM Confidential
 * OCO Source Materials
 * 5724-E68
 * 
 *  Copyright IBM Corp. 2003
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 * 
 */
package com.ibm.gre.dod.extension;

import java.io.IOException;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.EventObject;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

import javax.ejb.CreateException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.mail.Message.RecipientType;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
 
import com.ibm.gre.engine.ejb.facade.ClientControllerEJB;
import com.ibm.gre.engine.ejb.facade.FilePlanComponentControllerEJB;
import com.ibm.gre.engine.ejb.facade.FilePlanComponentControllerEJBLocal;
import com.ibm.gre.engine.ejb.facade.FilePlanComponentControllerEJBLocalHome;
import com.ibm.gre.engine.ejb.facade.LifeCycleDesignControllerEJBLocal;
import com.ibm.gre.engine.ejb.facade.LifeCycleDesignControllerEJBLocalHome;
import com.ibm.gre.engine.ejb.facade.NavigationControllerEJBLocalHome;
import com.ibm.gre.engine.ejb.facade.NavigationControllerEJBLocal;
import com.ibm.gre.engine.ejb.facade.LoginManagerEJB;
import com.ibm.gre.engine.ejb.facade.LoginManagerEJBHome;
import com.ibm.gre.engine.ejb.facade.QueryControllerEJB;
import com.ibm.gre.engine.ejb.facade.UserControllerEJB;
import com.ibm.gre.engine.value.StateMachine;
import com.ibm.gre.engine.extension.FilePlanComponentListener;
import com.ibm.gre.engine.util.BusinessComponentsJNDINames;
import com.ibm.gre.engine.util.BusinessLayerError;
import com.ibm.gre.engine.util.ComponentActionMaskConstant;
import com.ibm.gre.engine.util.DAOLayerErrors;
import com.ibm.gre.engine.util.DispositionTypes;
import com.ibm.gre.engine.util.ExtensionParamKeys;
import com.ibm.gre.engine.util.QueryResultOptions;
import com.ibm.gre.engine.util.RecordManagerException;
import com.ibm.gre.engine.util.RecordManagerExceptionTypes;
import com.ibm.gre.engine.util.ServiceLocator;
import com.ibm.gre.engine.util.ValidationUtil;
import com.ibm.gre.engine.util.XMLUtil;
import com.ibm.gre.engine.value.DBConfig;
import com.ibm.gre.engine.value.StateMachineKeys;
import com.ibm.gre.engine.value.User;
import com.ibm.gre.engine.options.DualNamingOptions;

/** 
 * @version 	1.0
 * @author		waisea
 * 
 * Purpose: purpose handle Dod document custom project permissions
 * Handles the following events:
 *	beforeAddEvent
 * 	afterAddEvent
 *	afterUpdateEvent
 * 	afterGetByIDEvent
 * 
 * IRM Configuration Requirements
 * ------------------------------
 * TS_User - must have TS_Project custom attribute of type String and multivalued enabled
 * optionally TS_Group too can have TS_Project custom attributes.
 * 
 * Custom component must have following custom attributes: 
 * 		1) TS_Project type String NON-multivalue droplist
 * 
 * The custom file plan component profile must contain both TS_Project custom attribute.
 * 
 * Within the package is a file called ApplicationResources.properties, it contains most
 * of the default values for the custom attriubtes stated above. In addition, you should be
 * able to modify the login user name and password using that file. This login user must be
 * administrator because it need to modify component's securities.
 * 
 * 
 * Building this project
 * ---------------------
 * 1) Make sure you have the lastest IRMAPI.jar and log4j-1.2.7.jar in your project root directory.
 * Change your Java Build Path Libraries to reference them.
 * 
 * 2) Export project as a jar, call it IRMDocumentExtension.jar include
 * package com.ibm.gre.dod.extension, do not include IRMAPI.jar and log4j-1.2.7.jar these
 * will be picked up from the engine class path when the class is loaded.
 * 
 * 
 *  Tips for running within IDE with Engine projects:
 *  ----------------
 * 	1) Modify Mocha (EAR) 
 * 		Deployment Descriptor - Module
 * 		Add (Project Utility JARs) to include IRMDocumentExtension.jar
 * 		Save it
 *  2) Modify MocharEJB MANIFEST.MF to include IRMDocumentExtension.jar as Dependencies
 *  3) To build IRMDocumentExtension, add project references to MochaEJB and MochaJava
 * 
 * ---------------- 
 * Revision History
 * ----------------
 * Date				Author		Changes
 * -------------------------------------
 * July 25, 2003		waisea		Created
 * Aug 26, 2003			waisea		Added beforeAddEvent handle new Dod case, and added external resource file for customization.
 * Sept 12, 2003		waisea		Get user controller as Administrator because calling userController.getGroupListByUserId requires User/Group function access
 * May 13, 2004			Tom Chen	Updated for Trafalgar.(i.e. removed the checking for Supplemental Markings)
 * 
 */
public class DocumentListener implements FilePlanComponentListener {
	// hard coded default user name and password
	private static String ADMIN_USER_NAME = "administrator";
	private static String ADMIN_PASSWORD = "cronos";
	// custom attributes Project and SuppMark required for User and FPC
	private static String CST_PROJECT = "TS_Project";

	// error constants	
	private static int E_EXTSN_BASE = 800000;
	private static int E_EXTSN_LOGIN_FAILED = E_EXTSN_BASE + 1; // failed login - required Administrator/cronos
	private static int E_EXTSN_NO_CLNT_CNTRLR = E_EXTSN_BASE + 2; // failed to create adminClientController
	private static int E_EXTSN_NO_USR_CNTRLR = E_EXTSN_BASE + 3; // failed to create user controller
	private static int E_EXSTN_BAD_XML = E_EXTSN_BASE + 4; // parsing xml failed.
	private static int E_EXSTN_NO_ACCESS_TO_FPC = E_EXTSN_BASE + 5;// no access to Fpc
	private static int E_EXSTN_NO_CST_PROJECT = E_EXTSN_BASE + 6;// missing CST_PROJECT in user
	//private static int E_EXSTN_NO_CST_SUPPMARK = E_EXTSN_BASE + 7;// missing CST_SUPPMARK in user
	private static int E_EXSTN_ALLOW_ACCESS = E_EXTSN_BASE + 8; // allowAccess failed - unknown exception
	private static int E_EXSTN_SET_PERMISSIONS = E_EXTSN_BASE + 9; // setPermissions failed
	private static int E_EXTSN_NO_QRY_CNTRLR = E_EXTSN_BASE + 10; // failed to create query controller
	
	
	private static int ACCESS_CHECK_ADD = 0;
	private static int ACCESS_CHECK_GETBYID = 1;

	// Application Resource Constants
	final private static String APP_RES_FILE_NAME = "ApplicationResources.properties";
	final private static String APP_RES_LOGIN_NAME = "login.user.name";
	final private static String APP_RES_LOGIN_PW = "login.user.pw";
	
	final private static String APP_RES_CST_PROJECT = "custom.field.project";
	final private static String APP_RES_CST_EMAIL_ADDRESS = "custom.field.email_address";
	final private static String APP_RES_CST_INFORM_ON_SUPERCEDE = "custom.field.inform_on_supercede";
	final private static String	APP_RES_MAIL_SMTP_HOST = "mail.smtp.host";
	final private static String	APP_RES_MAIL_FROM_ADDRESS = "mail.from.address";
	
	// Application Resouce Constants for error string
	private static String E_RES_LOGIN_BAD_USR_PW = "error.login.bad_user_pw";
	private static String E_RES_NO_ACCESS_ADD = "error.noaccess.add";
	private static String E_RES_NO_ACCESS_FETCH = "error.noaccess.fetch";
	private static String E_RES_NO_USR_CST_FIELD = "error.user.missing_custom_attribute";
	
	private Properties prop; // ApplicationResources.properties
	// loaded from application resources
	private String adminUserName;
	private String adminPassword;
	private String cstProject;
	private String cstEmailAddress;
	private String cstInformOnSupercede;
	
	private String mailSmtpHost;
	private String mailFromAddress;
	
	private InitialContext context = null ;
	
	// Project value from current fpc - assumed droplist
	private String fpcProjectValue ; 
	// component definition ID of the fpc
	private long fpcCmpntDefID ;
	
	// client controller that logs in as administrator - for changePermission operation
	private ClientControllerEJB adminClientController; 
	
	private static Logger logger = Logger.getLogger("EXTENSION");
	
	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeAddEvent(EventObject)
	 */
	public void beforeAddEvent(EventObject e) throws RecordManagerException {
		logger.info("beforeAddEvent - BEGIN");
		
		HashMap map = (HashMap) e.getSource();
		StateMachine stateMachine = (StateMachine) map.get(ExtensionParamKeys.STATE_MACHINE);
		
		// current user
		User user = (User)stateMachine.getAttribute(StateMachineKeys.USER);
		long userID = user.getUserID();

		if(-1 != userID){
			// skip permission check for user administrator
			String userName = user.getUserName();
			logger.info("beforeAddEvent - Current User [" + userName + "] ID [" + userID + "]");
					
			String fpcXml = (String) map.get(ExtensionParamKeys.FILE_PLAN_COMPONENT_XML);
//			logger.info("beforeAddEvent - fpcXML [" + fpcXml + "]");
			
			initApplicationResources();		
			accessAllowed(stateMachine,userID,fpcXml,ACCESS_CHECK_ADD);
		}		
		logger.info("beforeAddEvent - END");		
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterAddEvent(EventObject)
	 */
	public void afterAddEvent(EventObject e) throws RecordManagerException {
		logger.info("afterAddEvent - BEGIN");
		
		HashMap map = (HashMap) e.getSource();
		StateMachine stateMachine = (StateMachine) map.get(ExtensionParamKeys.STATE_MACHINE);
		
		long cmpntID = ((Long) map.get(ExtensionParamKeys.ADD)).longValue();
		String fpcXml = (String) map.get(ExtensionParamKeys.FILE_PLAN_COMPONENT_XML);
		fpcCmpntDefID = ((Long) map.get(ExtensionParamKeys.FILE_PLAN_COMPONENT_DEFINITION_ID)).longValue();

//		logger.info("afterAddEvent - New cmpntID " + cmpntID + " cmpntDefID " + fpcCmpntDefID);
//		logger.info("afterAddEvent - fpcXml " + fpcXml);
		
		initApplicationResources();
		setPermissions(stateMachine,cmpntID,fpcXml,false);
		
		logger.info("afterAddEvent - END");
				
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeDeleteEvent(EventObject)
	 */
	public void beforeDeleteEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterDeleteEvent(EventObject)
	 */
	public void afterDeleteEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeGetByBarCodeEvent(EventObject)
	 */
	public void beforeGetByBarCodeEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterGetByBarCodeEvent(EventObject)
	 */
	public void afterGetByBarCodeEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeUpdateEvent(EventObject)
	 */
	public void beforeUpdateEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterUpdateEvent(EventObject)
	 */
	public void afterUpdateEvent(EventObject e) throws RecordManagerException {
		logger.info("afterUpdateEvent - BEGIN");
		
		HashMap map = (HashMap) e.getSource();
		StateMachine stateMachine = (StateMachine) map.get(ExtensionParamKeys.STATE_MACHINE);

		long cmpntID = ((Long) map.get(ExtensionParamKeys.FILE_PLAN_COMPONENT_ID)).longValue();
		fpcCmpntDefID = ((Long) map.get(ExtensionParamKeys.FILE_PLAN_COMPONENT_DEFINITION_ID)).longValue();
		logger.info("afterUpdateEvent - cmpntID " + cmpntID + " cmpntDefID " + fpcCmpntDefID);		
		String fpcXml = (String) map.get(ExtensionParamKeys.FILE_PLAN_COMPONENT_XML);		
//		logger.info("afterUpdateEvent - fpcXml " + fpcXml);
		
		initApplicationResources();		
		setPermissions(stateMachine,cmpntID,fpcXml,true);
		
		logger.info("afterUpdateEvent - END");				
		
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeGetByIDEvent(EventObject)
	 */
	public void beforeGetByIDEvent(EventObject e) throws RecordManagerException {
		//logger.info("starting beforeGetByIDEvent ...");
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterGetByIDEvent(EventObject)
	 */
	public void afterGetByIDEvent(EventObject e) throws RecordManagerException {
		logger.info("afterGetByIDEvent - BEGIN");
		
		HashMap map = (HashMap) e.getSource();
		StateMachine stateMachine = (StateMachine) map.get(ExtensionParamKeys.STATE_MACHINE);
		
		// current user
		User user = (User)stateMachine.getAttribute(StateMachineKeys.USER);
		long userID = user.getUserID();

		if(-1 != userID){
			// skip permission check for user administrator
			String userName = user.getUserName();
			logger.info("afterGetByIDEvent - Current User [" + userName + "] ID [" + userID + "]");
					
			String fpcXml = (String) map.get(ExtensionParamKeys.GET_BY_ID);
//			logger.info("afterGetByIDEvent - fpcXML [" + fpcXml + "]");
			
			initApplicationResources();		
			accessAllowed(stateMachine,userID,fpcXml,ACCESS_CHECK_GETBYID);
		}		
		logger.info("afterGetByIDEvent - END");
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeSetFieldEvent(EventObject)
	 */
	public void beforeSetFieldEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterSetFieldEvent(EventObject)
	 */
	public void afterSetFieldEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeGetFieldEvent(EventObject)
	 */
	public void beforeGetFieldEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterGetFieldEvent(EventObject)
	 */
	public void afterGetFieldEvent(EventObject e)
		throws RecordManagerException {
	}
	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterTransitionEvent(EventObject)
	 */
	public void afterTransitionEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeTransitionEvent(EventObject)
	 */
	public void beforeTransitionEvent(EventObject e) throws RecordManagerException {
	}
	
	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeSupercedeEvent(java.util.EventObject)
	 */
	public void beforeSupercedeEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterSupercedeEvent(java.util.EventObject)
	 */
	public void afterSupercedeEvent(EventObject e) throws RecordManagerException {
		logger.info("afterSupercedeEvent - BEGIN");
		
		//retrieve the parameters
		HashMap map = (HashMap) e.getSource();
		StateMachine stateMachine = (StateMachine) map.get(ExtensionParamKeys.STATE_MACHINE);

		long supercededFPCId = ((Long) map.get(ExtensionParamKeys.SUPERCEDED_FILE_PLAN_COMPONENT_ID)).longValue();
		logger.info("supercededFPCId " + supercededFPCId);
		
		long supercedingFPCId = ((Long) map.get(ExtensionParamKeys.SUPERCEDING_FILE_PLAN_COMPONENT_ID)).longValue();
		logger.info("supercedingFPCId " + supercedingFPCId);
		
		Date supercedeDate = (Date) map.get(ExtensionParamKeys.SUPERCEDE_DATE);
		logger.info("supercedeDate " + supercedeDate);

		long viewId = ((Long) map.get(ExtensionParamKeys.VIEW_ID)).longValue();
		logger.info("viewId " + viewId);

		boolean isLifeCycleBegin = ((Boolean) map.get(ExtensionParamKeys.IS_LIFECYCLE_BEGIN)).booleanValue();
		logger.info("isLifeCycleBegin " + isLifeCycleBegin);
		
		//get full path of the superceded fpc and superceding fpc.
		NavigationControllerEJBLocalHome nch = null;
		NavigationControllerEJBLocal ncl = null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.NAVIGATION_CONTROLLER_JNDI);
	
			nch = (NavigationControllerEJBLocalHome)PortableRemoteObject.narrow(obj, NavigationControllerEJBLocalHome.class);
	
			ncl = nch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		String supercededFPCFullPathXML = ncl.getFullPathListById(supercededFPCId, DualNamingOptions.BY_NAME);
		String supercedingFPCFullPathXML = ncl.getFullPathListById(supercedingFPCId, DualNamingOptions.BY_NAME);
		
		Document supercededFPCFullPathDoc = XMLUtil.getDocument(supercededFPCFullPathXML);
		Document supercedingFPCFullPathDoc = XMLUtil.getDocument(supercedingFPCFullPathXML);
		String supercededFPCFullPath = XMLUtil.getStringValue(
				XMLUtil.getNodeByXPath(supercededFPCFullPathDoc,"/List/Item/@FullPath"));
		String supercedingFPCFullPath = XMLUtil.getStringValue(
				XMLUtil.getNodeByXPath(supercedingFPCFullPathDoc,"/List/Item/@FullPath"));
		
		//get the life cycle code of the superceded fpc
		FilePlanComponentControllerEJBLocalHome fch =null;
		FilePlanComponentControllerEJBLocal fcl = null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.FILE_PLAN_COMPONENT_CONTROLLER_JNDI);
	
			fch = (FilePlanComponentControllerEJBLocalHome) PortableRemoteObject.narrow(obj, FilePlanComponentControllerEJBLocalHome.class);
	
			fcl = fch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		
		String lccField = fcl.getFilePlanComponentField(supercededFPCId,"LfCclCdID");
		
		if(ValidationUtil.isEmpty(lccField) || lccField.equals("0")){
			logger.info("Life Cycle Code of the superceded FPC is empty or 0. No E-mail is sent.");
			logger.info("afterSupercedeEvent - END");
			return;
		}
		long codeId = Long.parseLong(lccField);
		
		//get disposal type and beginLifeCycleWhenSuperceded
		LifeCycleDesignControllerEJBLocalHome lcdch = null;
		LifeCycleDesignControllerEJBLocal lcdcl =null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.LIFE_CYCLE_DESIGN_CONTROLLER_JNDI);
	
			lcdch = (LifeCycleDesignControllerEJBLocalHome) PortableRemoteObject.narrow(obj, LifeCycleDesignControllerEJBLocalHome.class);
	
			lcdcl = lcdch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		
		String lifeCycleXML = lcdcl.getLifeCycleManagementCodeById(codeId);		
		Document lifeCycleDoc = XMLUtil.getDocument(lifeCycleXML);
		
		String supercedeValue = XMLUtil.getNodeValue(lifeCycleDoc, "Field", "FlPlnCmpntAttrDefColNm", 
									"BgnLfCclWhenSprsd");
		String disposalTypeValue = XMLUtil.getNodeValue(lifeCycleDoc, "Field", "FlPlnCmpntAttrDefColNm", 
									"DspTyp");

		boolean isBeginLCCWhenSuperceded =false;
		if (supercedeValue !=null && supercedeValue.equals("1")) {
			isBeginLCCWhenSuperceded = true;
		}
		
		long disposalType = Long.parseLong(disposalTypeValue);
		
		if (isBeginLCCWhenSuperceded && disposalType == DispositionTypes.DESTROY) {
			
			
			initApplicationResources();
			String userQuery = buildUsersQuery(cstEmailAddress,cstInformOnSupercede);
			loginAsAdministrator(stateMachine);
			
			//get users to send email to
			QueryControllerEJB qc = getQueryController(); 
			
			String userResult = null;
			try { 
				userResult = qc.runQuery(userQuery, QueryResultOptions.NORMAL);
			} catch(Exception exp) {
				throw new RecordManagerException(BusinessLayerError.UNKNOWN, exp.getMessage(),getClass().getName(),
					RecordManagerExceptionTypes.SYSTEM, exp);
			}

			Document domDoc = XMLUtil.getDocument(userResult);	
			NodeList nodeList = XMLUtil.getNodeListByXPath(domDoc,"//ResultList/Data/List/Item");
			String [] emailAddress ;
			int nLen = nodeList.getLength();
			emailAddress = new String[nLen];

			String usrCstEmailAddress = "TS_UsrDir_Cst." + cstEmailAddress;
			for (int i = 0; i < nLen ; i++){
				Node nodeItem = nodeList.item(i);
				emailAddress[i] = XMLUtil.getStringValueFromXMLAttribute(nodeItem,usrCstEmailAddress);
			}
			
			StringBuffer body = new StringBuffer();
			body.append(supercededFPCFullPath).append(" (id:").append(supercededFPCId).append(")").
				append(" is superceded in a category that has a final disposition of " +
					"\"Destroy when superseded or obsolete.\" by ").
				append(supercedingFPCFullPath).append(" (id:").append(supercedingFPCId).append(") ");
			
			if (emailAddress.length==0) {
				logger.info("No E-mail address is configured. No E-mail is sent.");
				logger.info("This Message should have sent if E-mail address is configured:"+ body.toString());
				logger.info("afterSupercedeEvent - END");
				return;
			}
			
			sendEmail(mailFromAddress, emailAddress, "Supercede", body.toString());
		}
		

		logger.info("afterSupercedeEvent - END");
	}

	private String buildUsersQuery (String userEmailAddress, String userInformOnSupercede) {
		StringBuffer sbQuery = new StringBuffer();
		// -3 for user
		sbQuery.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		sbQuery.append("<Fields><Field FlPlnCmpntAttrDefColNm=\"FlPlnCmpntDefID\">-3</Field>");
		sbQuery.append("<Field FlPlnCmpntAttrDefColNm=\"QueryImage\">");
		// ReportTemplate - Start
		sbQuery.append("<![CDATA[<ReportTemplate>");
		// parameters
		sbQuery.append("<Version>1.0</Version>");
		sbQuery.append("<ReportLayout />");
		sbQuery.append("<CountOnly>0</CountOnly>");
		sbQuery.append("<Scope />"); 
		sbQuery.append("<BlockSize>0</BlockSize>");
		sbQuery.append("<BlockStart>0</BlockStart>");
  		
		// Query - Start
		sbQuery.append("<Query>");
  		
		// Condition - Start
		sbQuery.append("<Condition>");
  		
  		
		sbQuery.append("<FieldExpression>");
		sbQuery.append("<FieldName>TS_UsrDir_Cst." + userInformOnSupercede + "</FieldName>");
		sbQuery.append("<FieldOperator>=</FieldOperator>");
		sbQuery.append("<FieldValue>1</FieldValue>");
		sbQuery.append("</FieldExpression>");
  		
		sbQuery.append("<InterFieldOperator>AND</InterFieldOperator>");
  		
		sbQuery.append("<InterFieldOperator>NOT</InterFieldOperator>");
		sbQuery.append("<FieldExpression>");
		sbQuery.append("<FieldName>TS_UsrDir_Cst." + userEmailAddress + "</FieldName>");
		sbQuery.append("<FieldOperator>IS NULL</FieldOperator>");
		sbQuery.append("<FieldValue/>");
		sbQuery.append("</FieldExpression>");  		
    		
		// Condition - End
		sbQuery.append("</Condition>");  
		// SortFields
		sbQuery.append("<SortFields />");
		// OutputFields - Begin
		sbQuery.append("<OutputFields>");
		sbQuery.append("<OutputField>TS_UsrDir_Cst." + userEmailAddress + "</OutputField>");
		sbQuery.append("<OutputField>TS_UsrDir.UsrNm</OutputField>");
		// OutputFields - End
		sbQuery.append("</OutputFields>");
  		
		// Query - End
		sbQuery.append("</Query>");
  		  		
		// ReportTemplate - End		
		sbQuery.append("</ReportTemplate>]]>");
		sbQuery.append("</Field></Fields>");
		
		return sbQuery.toString();		
	}



	/**
	 * 
	 */
	private void sendEmail(String fromAddress,String[] toAddress, String subject, String body) 
		throws RecordManagerException {
		Properties props = new Properties ();
		props.put("mail.smtp.host", mailSmtpHost);
		logger.info("smtp host:" + mailSmtpHost);   
		try {
			
			Session sendMailSession = Session.getDefaultInstance(props);
	   		
			MimeMessage mailMessage = new MimeMessage(sendMailSession);
			mailMessage.setSubject(subject);
	
			mailMessage.setText(body);
	
	
			InternetAddress addressFrom = new InternetAddress(fromAddress);
			mailMessage.setFrom(addressFrom);
	   		
			for (int i=0 ; i < toAddress.length ; i++){
				mailMessage.addRecipient(RecipientType.TO ,new InternetAddress(toAddress[i]));
			}
			java.util.Enumeration enum = mailMessage.getAllHeaderLines();
			while(enum.hasMoreElements()){
				Object obj = enum.nextElement();
				logger.info(obj.toString());
			}
			
			Transport.send(mailMessage);
			
			
		} catch (Exception e) {
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, e.getMessage(), 
				getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}
		logger.info("E-mail is sent:"+body);
		
	}

	/**
	 * Method initApplicationResources.
	 * This function must be called before all events. It will initialize requires login/pw
	 * project and suppMark from ApplicationResources.
	 * 
	 */
	private void initApplicationResources () {
		logger.info("initApplicationResources - BEGIN");
		InputStream is = DocumentListener.class.getResourceAsStream(APP_RES_FILE_NAME);
		prop = new Properties();
		if(null != is){
			try{
				prop.load(is);
			}
			catch(IOException e){
				logger.info("initApplicationResources - Properties.load failed ",e);
			}
		}
		else{
			logger.info("initApplicationResources - getResoucesAsStream is null");
		}
		adminUserName = prop.getProperty(APP_RES_LOGIN_NAME,ADMIN_USER_NAME);
		adminPassword = prop.getProperty(APP_RES_LOGIN_PW,ADMIN_PASSWORD);
		
		cstProject = prop.getProperty(APP_RES_CST_PROJECT,CST_PROJECT);
		cstEmailAddress = prop.getProperty(APP_RES_CST_EMAIL_ADDRESS);
		cstInformOnSupercede = prop.getProperty(APP_RES_CST_INFORM_ON_SUPERCEDE);
		mailSmtpHost = prop.getProperty(APP_RES_MAIL_SMTP_HOST);
		mailFromAddress = prop.getProperty(APP_RES_MAIL_FROM_ADDRESS);
		
		logger.info("initApplicationResources - END");		
	}
	private String getMessage(String key){
		return prop.getProperty(key,key);
	}
	private String getMessage(String key, String arg0){
		return getMessage (key,arg0,null,null);
	}	
	private String getMessage(String key, String arg0, String arg1){
		return getMessage(key,arg0,arg1,null);
	}
	private String getMessage(String key, String arg0, String arg1, String arg2){
		String msg = prop.getProperty(key,key);
		String formatedMsg ;
		try{
			String [] args = {arg0, arg1, arg2};
			formatedMsg = MessageFormat.format(msg,args);
		}
		catch(IllegalArgumentException e){
			formatedMsg = key;
		}
		return formatedMsg;
	}	
	/**
	 * creates adminClientController for permission modification
	 * 
	 * Method loginAsAdministrator.
	 * @param stateMachine
	 * @throws RecordManagerException
	 */
	private void loginAsAdministrator(StateMachine stateMachine) throws RecordManagerException{

		try{
			logger.info("loginAsAdministrator - Before getLoginManagerEJBHome ...");
			if(null == context){
				context = new InitialContext ();
			}
			Object obj = context.lookup(BusinessComponentsJNDINames.LOGIN_MANAGER_EJB_JNDI);
			LoginManagerEJBHome loginMgrHome = (LoginManagerEJBHome) PortableRemoteObject.narrow(obj, LoginManagerEJBHome.class);

//			logger.info("loginAsAdministrator - after getLoginManagerEJBHome ...");
						
			LoginManagerEJB loginMgr = loginMgrHome.create();
//			logger.info("loginAsAdministrator - Before getDatasource ...");
			DBConfig dbConfig = (DBConfig)stateMachine.getAttribute(StateMachineKeys.DATASOURCE);
			logger.info("loginAsAdministrator - Before login Datasource [" + dbConfig.jndi_name + "] User [" + adminUserName + "]");
			adminClientController = loginMgr.login(adminUserName,adminPassword,dbConfig.jndi_name,"","en");
			loginMgr.remove();
			logger.info("loginAsAdministrator - DONE! ...");
		}
		catch(Exception e){
			logger.info("loginAsAdministrator - caught exception!",e);
			if(e instanceof RecordManagerException ){
				RecordManagerException rme = (RecordManagerException) e;
				if(DAOLayerErrors.USER_NOT_FOUND == rme.getErrorKey()){
					throw new RecordManagerException(E_EXTSN_LOGIN_FAILED, getMessage(E_RES_LOGIN_BAD_USR_PW), getClass().getName(), RecordManagerExceptionTypes.SYSTEM,rme);					
				}
				throw rme;
			}
			else{
				throw new RecordManagerException(E_EXTSN_NO_CLNT_CNTRLR, e.getMessage(), this.getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
			}
		}
	}
	
	private UserControllerEJB getUserController(StateMachine stateMachine) throws RecordManagerException{
		try{
//			logger.info("getUserController - locate UserControllerEJBLocalHome");
/*			if(null == context){
				context = new InitialContext ();
			}
			Object obj = context.lookup("local:ejb/" + BusinessComponentsJNDINames.USER_CONTROLLER_JNDI);
			UserControllerEJBLocalHome uch = (UserControllerEJBLocalHome) PortableRemoteObject.narrow(obj, UserControllerEJBLocalHome.class);
			
//			logger.info("getUserController - UserControllerEJBLocal.create");			
			UserControllerEJBLocal uc = uch.create(stateMachine);			
*/
			// need userController with Administrator because we need call function that requires Security Management function access
			UserControllerEJB uc = adminClientController.getUserController();
			return uc;			
		}
		catch(Exception e){
			logger.info ("getUserController - caught Exception ", e);			
			throw new RecordManagerException(E_EXTSN_NO_USR_CNTRLR, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}
	}

	private QueryControllerEJB getQueryController() throws RecordManagerException{
		try{
			// need queryController with Administrator because we need call function that requires Security Management function access
			QueryControllerEJB qc = adminClientController.getQueryController();
			return qc;			
		}
		catch(Exception e){
			logger.info ("getUserController - caught Exception ", e);			
			throw new RecordManagerException(E_EXTSN_NO_QRY_CNTRLR, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}
	}

	
	private void accessAllowed (StateMachine stateMachine, long usrID, String fpcXml, int mode) throws RecordManagerException {
		
		boolean hasAttributes = false;
		UserControllerEJB  userController = null;
		try{
			fetchFpcProject(fpcXml);
			loginAsAdministrator(stateMachine);
			
			if(fpcProjectValue.length() > 0){
				userController = getUserController(stateMachine);
				hasAttributes = userHasAttribute(userController,usrID);
			}
			else{
				// let view permission take care of this
				hasAttributes = true;
				logger.info("accessAllowed - both Project and SuppMarks are empty in fpcXml");
			}
		}
		catch(RecordManagerException rme){
			logger.info("accessAllowed - caught RME EXCEPTION ",rme);
			throw rme;
		}
		catch(Exception e){
			logger.info("accessAllowed - caught UNKNOWN EXCEPTION ",e);
			throw new RecordManagerException(E_EXSTN_ALLOW_ACCESS, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM,e);				
		}
		/*
		finally{
			try{
				if(null != userController){				
					userController.remove();
				}
				if(null != adminClientController){
					adminClientController.remove();
				}				
			}
			catch(Exception e){
				logger.info("accessAllowed - caught cleanUp Exception ",e);
			}			
		} */
		
		logger.info("accessAllowed - hasAttributes " + hasAttributes);
		if (!hasAttributes){
			logger.info("accessAllowed - throw Exception NO access to file plan component");
			if(ACCESS_CHECK_ADD == mode){
				throw new RecordManagerException(E_EXSTN_NO_ACCESS_TO_FPC, getMessage(E_RES_NO_ACCESS_ADD), getClass().getName(), RecordManagerExceptionTypes.SYSTEM);
			}
			else {
				// default for ACCESS_CHECK_GETBYID
				throw new RecordManagerException(E_EXSTN_NO_ACCESS_TO_FPC, getMessage(E_RES_NO_ACCESS_FETCH), getClass().getName(), RecordManagerExceptionTypes.SYSTEM);
			}
		}
	}
	
	/**
	 * This function will populate fpcProjectValue and fpcSuppMarkValues from fpcXml.
	 * if there is no Project in fpcXml fpcProjectValue will empty string similarly fpcSuppMarkValues will
	 * set to empty arrayList if there is no SuppMark value.
	 * 
	 * Method fetchFpcSuppMarkAndProject.
	 * @param fpcXml
	 * @throws RecordManagerException
	 */
	private void fetchFpcProject(String fpcXml) throws RecordManagerException{
		Document fpcDomDoc = XMLUtil.getDocument(fpcXml);
		
		
		String searchPattern = "//Fields/Field[@FlPlnCmpntAttrDefColNm = \"" + cstProject + "\"]";
		Node fieldNode = XMLUtil.getNodeByXPath(fpcDomDoc,searchPattern);
		if(null != fieldNode){
			try{
				fpcProjectValue = XMLUtil.getValue(fieldNode);
				if(null == fpcProjectValue){
					fpcProjectValue = "";
				}
			}
			catch(Exception e){
				logger.info("fetchFpcSuppMarkAndProject - caught Exception getValue for " + cstProject);
				throw new RecordManagerException(E_EXSTN_BAD_XML, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);				
			}
		}
		else{
			fpcProjectValue = "";			
		}
		logger.info("fetchFpcSuppMarkAndProject - Fpc Project [" + fpcProjectValue + "]");
		
	}
	private boolean userHasAttribute (UserControllerEJB userController, long userID) throws Exception {
		logger.debug ("userHasAttribute - getUserById(" + userID + ")");
		// we may need to exit for Administrator user here
		String userOrGroupXml = userController.getUserById(userID,false);
		logger.info ("userHasAttribute - userXml [" + userOrGroupXml + "]");
		Document userOrGroupDom = XMLUtil.getDocument(userOrGroupXml);
	
		ArrayList projectList = new ArrayList();
		
		buildUnionOfProjects(projectList,userOrGroupDom,true);
		
		// find groups this user belongs to
		String groupListXml = userController.getGroupListByUserId(userID);
		Document groupListDom = XMLUtil.getDocument(groupListXml);
		groupListXml = null;
		NodeList groupNodeList = XMLUtil.getNodeListByXPath(groupListDom,"//List/Item");
		for(int i=0; i < groupNodeList.getLength(); i++){
			Node groupNodeItem = groupNodeList.item(i);
			long groupID = Long.parseLong(getNodeAttribute(groupNodeItem,"GrpID"));
			// we may need to exit for Administrator group here
			userOrGroupXml = userController.getGroupById(groupID,false);
			userOrGroupDom = XMLUtil.getDocument(userOrGroupXml);
			// combine the projects and suppmark
			buildUnionOfProjects(projectList,userOrGroupDom,false);	
		}
		groupNodeList = null;
		groupListDom = null;
		userOrGroupXml = null;
	
		// fpcProjectValue NOT in user's Project list
		if(!isValueInSource(projectList,fpcProjectValue)){
			logger.info("userHasAttribute - fpcProjectValue does not match User's project list");
			return false;
		}
		logger.info("userHasAttribute - fpcProjectValue matched User's project list");			
		
		return true;
	}
	private void buildUnionOfProjects(ArrayList projectList, Document userOrGroupDom, 
					boolean isUser) throws RecordManagerException {
	
		String searchPattern = "//Fields/Field[@FlPlnCmpntAttrDefColNm = \"" + cstProject + "\"]";
		NodeList nodeList = XMLUtil.getNodeListByXPath(userOrGroupDom,searchPattern);
		if(isUser && (null == nodeList || nodeList.getLength() == 0)){
			logger.info ("buildUnionOfProjectsAndSuppmark - throw Exception! TS_User missing attribute " + cstProject);
			throw new RecordManagerException(E_EXSTN_NO_CST_PROJECT, getMessage(E_RES_NO_USR_CST_FIELD,cstProject), getClass().getName(), RecordManagerExceptionTypes.SYSTEM);
		}
		
		searchPattern = "//Fields/Field[@FlPlnCmpntAttrDefColNm = \"" + cstProject + "\"]/v";
		nodeList = XMLUtil.getNodeListByXPath(userOrGroupDom,searchPattern);
		if(null != nodeList){
			appendMultiValuedList(nodeList,projectList);
		}
		
	}
	private ArrayList buildMultiValuedList (NodeList nodeList) throws RecordManagerException{
		int nLen = nodeList.getLength();
		ArrayList arrayOfValues = new ArrayList(nLen);
		for(int i=0; i < nLen ; i++){
			Node nodeItem = nodeList.item(i);
			String txtData = null;
			try{				
				txtData = XMLUtil.getValue(nodeItem);
				if(txtData != null && txtData.length() > 0){
					arrayOfValues.add(txtData);
				}					
			}
			catch(Exception e){
				throw new RecordManagerException(E_EXSTN_BAD_XML, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);				
			}
		}
		return arrayOfValues;		
	}
	
	private void appendMultiValuedList (NodeList nodeList, ArrayList arrayOfValues) throws RecordManagerException{
		int nLen = nodeList.getLength();
		
		for(int i=0; i < nLen ; i++){
			Node nodeItem = nodeList.item(i);
			String txtData = null;
			try{				
				txtData = XMLUtil.getValue(nodeItem);
				if(txtData != null && txtData.length() > 0){
					if(!isValueInSource(arrayOfValues,txtData)){
						arrayOfValues.add(txtData);
					}
				}					
			}
			catch(Exception e){
				throw new RecordManagerException(E_EXSTN_BAD_XML, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);				
			}
		}
	}	
	private boolean isValueInSource(ArrayList sourceList, String value) {
		if(null == sourceList || null == value){
			return false;
		}
		// nothing to check on the document against the project or suppmark list
		if(value.length() == 0){
			return true;
		}
		Iterator it = sourceList.iterator();
		while(it.hasNext()){
			String sourceValue = (String) it.next();
			if(sourceValue.equals(value)){
				return true;
			}
		}
		return false;
	}
	private boolean isAllValuesInSource(ArrayList sourceList, ArrayList valueList){
		if(null == valueList){
			return false;
		}
		// all valueList must be in source
		Iterator it = valueList.iterator();
		while(it.hasNext()){
			String value = (String) it.next();
			if(!isValueInSource(sourceList,value)){
				return false;
			}
		}
		return true;		
	}
	
	private void setPermissions(StateMachine stateMachine, long cmpntID, String fpcXml, boolean isUpdateEvent) throws RecordManagerException {
		
		FilePlanComponentControllerEJB fpcController = null;
		UserControllerEJB userController = null;
		try{
			
			fetchFpcProject(fpcXml);
			
			loginAsAdministrator(stateMachine);			

			logger.info("setPermissions - getFilePlanComponentController");
			fpcController = adminClientController.getFilePlanComponentController();
			logger.info("setPermissions - getFilePlanComponetController DONE!");			
			
			if(fpcProjectValue.length() == 0){
				if(isUpdateEvent){
					// on addEvent is always inheriting from parent permission
					logger.info("setPermissions - inherit from parent");
					fpcController.inheritFromParent(cmpntID,false);
				}
				return;
			}
			logger.info("setPermissions - getPermissionList(" + cmpntID + ",-1," + fpcCmpntDefID + ")");			
			String filter = "<Filter><IRMFilter FlPlnCmpntAttrDefColNm='RecHsID'>-1</IRMFilter></Filter>";
			
			String fpcPermissionXml = fpcController.getPermissionList(cmpntID, fpcCmpntDefID, filter, 0, 0);
			logger.info("setPermissions - permissionList " + fpcPermissionXml );
			
			Document permissionDomDoc = XMLUtil.getDocument(fpcPermissionXml);
			
			NodeList permissionNodeList = XMLUtil.getNodeListByXPath(permissionDomDoc,"//List/Item");
			if(null == permissionNodeList || permissionNodeList.getLength() == 0){
				return;
			}
			
			userController = getUserController(stateMachine);
			modifyPermissionWithGroups(userController,permissionDomDoc);
			
			// refresh nodeList because modifyPermissionWithGroups play with permissionDomDoc
			permissionNodeList = XMLUtil.getNodeListByXPath(permissionDomDoc,"//List/Item");
			
			changePermissions (cmpntID,permissionNodeList,fpcController,userController);
			
		}
		catch(RecordManagerException rme){
			logger.info("setPermissions - caught RME EXCEPTION ",rme);
			throw rme;
		}
		catch(Exception e){
			logger.info("setPermissions - caught UNKNOWN EXCEPTION ",e);
			throw new RecordManagerException(E_EXSTN_SET_PERMISSIONS, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM,e);				
		}
		/*
		finally{
			try{

				if(null != fpcController){
					fpcController.remove();
				}
				if(null != userController){
					userController.remove();
				}				
				if(null != adminClientController){
					adminClientController.remove();
				}
				
				
			}
			catch(Exception e){
				logger.info("setPermissions - cleanup caught Exception ",e);
			}
		} */

		
	}
	
	private void modifyPermissionWithGroups(UserControllerEJB userController, Document permissionDomDoc) throws Exception{
		logger.info("modifyPermissionWithGroups - BEGIN");
		// get list of permission belonging to groups
		NodeList nodeList = XMLUtil.getNodeListByXPath(permissionDomDoc,"//List/Item[@UsrGrpTyp=\"2\"]");
		if(null != nodeList && nodeList.getLength() > 0){
			for(int i=0; i < nodeList.getLength() ; i++){
				Node nodeItem = nodeList.item(i);
				NamedNodeMap nodeAttributes = nodeItem.getAttributes();
				String temp = getNamedNodeMapAttribute(nodeAttributes,"UsrGrpID");
				long grpID = Long.parseLong(temp);
				temp = getNamedNodeMapAttribute(nodeAttributes,"FlPlnCmpntActnMsk");
				int actnMsk = Integer.parseInt(temp);
				logger.info("modifyPermissionWithGroups - found Group " + grpID + " ActnMsk " + actnMsk); 
				if((actnMsk & ComponentActionMaskConstant.FETCH) == ComponentActionMaskConstant.FETCH){
					// remove view bit from group
					actnMsk &= ~(ComponentActionMaskConstant.FETCH);
					((Element) nodeItem).setAttribute("FlPlnCmpntActnMsk",String.valueOf(actnMsk));
					//retrieve users from all hosts and local.
					String filter = "<Filter><IRMFilter FlPlnCmpntAttrDefColNm='RecHsID'>-1</IRMFilter></Filter>";
					String membersXml = userController.getUserListByGroupId(grpID, filter, 0 , 0);
//					logger.info("modifyPermissionWithGroups - membersXml " + membersXml);
					addGroupMembersToPermission(permissionDomDoc,membersXml);
				}
			}
		}
		logger.info("modifyPermissionWithGroups - END");
	}

	private String getNamedNodeMapAttribute(NamedNodeMap nodeAttributes, String attributeName){
		Node attributeNode = nodeAttributes.getNamedItem(attributeName);
		
		if(null == attributeNode){
			// bad input data
			return null;
		}
		return attributeNode.getNodeValue();
	}
	
	private String getNodeAttribute(Node nodeItem, String attribute) {	
		NamedNodeMap nodeAttributes = nodeItem.getAttributes();
		return getNamedNodeMapAttribute (nodeAttributes,attribute);
	}	
	
	private void addGroupMembersToPermission(Document permissionDomDoc, String membersXml) throws Exception{

		logger.info("addGroupMembersToPermission - BEGIN");
		Document membersDomDoc = XMLUtil.getDocument(membersXml);
		NodeList membersNodeList = XMLUtil.getNodeListByXPath(membersDomDoc,"//List/Item");
		if(null == membersNodeList || membersNodeList.getLength() == 0){
			logger.info("addGroupMembersToPermission - END (empty memberList)");
			return ;
		}
		
		String strFpcDefID = String.valueOf(fpcCmpntDefID);
		Element nodeRootPermission = permissionDomDoc.getDocumentElement();
		
		for (int i=0; i < membersNodeList.getLength(); i++){
			Node nodeItem = membersNodeList.item(i);
			String usrID = getNodeAttribute(nodeItem,"UsrID");
			
			Node userNode = XMLUtil.getNodeByXPath(permissionDomDoc,"//List/Item[@UsrGrpID=\"" + usrID + "\"]");
			if(null != userNode){
				// in Permission list - modify view permission 'inherit by group'
				String temp = getNodeAttribute(userNode,"FlPlnCmpntActnMsk");
				int actnMsk = Integer.parseInt(temp);
				actnMsk |= ComponentActionMaskConstant.FETCH ;
				((Element) userNode).setAttribute("FlPlnCmpntActnMsk",String.valueOf(actnMsk));
				logger.info("addGroupMembersToPermission - user " + usrID + " in permissionList change actnMsk (" + temp + " -> " + String.valueOf(actnMsk) + ")");				
			}
			else{
				// not in Permssion list, add with View only permission 'inherit by group'
				logger.info("addGroupMembersToPermission - 	user NOT in permissionList add new entry to permissionList");
				Element nodePerm = permissionDomDoc.createElement("Item");
				
	           	nodePerm.setAttribute("FlPlnCmpntActnMsk", "8");
	            nodePerm.setAttribute("FlPlnCmpntDefID", strFpcDefID);
	            nodePerm.setAttribute("UsrGrpID", usrID);
	            nodePerm.setAttribute("UsrGrpTyp","1");
	            
	            nodeRootPermission.appendChild(nodePerm);				
			}
		}
		logger.info("addGroupMembersToPermission - END");		
	}
	
	private void changePermissions(long cmpntID, NodeList permissionNodeList, FilePlanComponentControllerEJB fpcController,UserControllerEJB userController) throws Exception {
		logger.info("changePermissions - BEGIN");
		String strFpcDefID = String.valueOf(fpcCmpntDefID);
		StringBuffer sbPermission = new StringBuffer();
		sbPermission.append("<List>");
		for (int i=0; i < permissionNodeList.getLength() ; i++){
			Node nodeItem = permissionNodeList.item(i);
			NamedNodeMap nodeAttributes = nodeItem.getAttributes();
			String usrGrpID = getNamedNodeMapAttribute(nodeAttributes,"UsrGrpID");
			String strActnMsk = getNamedNodeMapAttribute(nodeAttributes,"FlPlnCmpntActnMsk");
			String usrGrpTyp = getNamedNodeMapAttribute(nodeAttributes,"UsrGrpTyp");
			
			if("1".equals(usrGrpTyp)){ // User
				// New for 3.1 group is separated therefore do not have supplemental marking
				int actnMsk = Integer.parseInt(strActnMsk);
				
				if(userHasAttribute(userController,Long.parseLong(usrGrpID))){
					// set view bit
					actnMsk |= ComponentActionMaskConstant.FETCH;
				}
				else{
					// remove view bit
					actnMsk &= ~(ComponentActionMaskConstant.FETCH);
				}
				strActnMsk = String.valueOf(actnMsk);
			}
			sbPermission.append("<Item UsrGrpID=\"" + usrGrpID + "\" FlPlnCmpntDefID=\"" + strFpcDefID + "\" ");
			sbPermission.append("FlPlnCmpntActnMsk=\"" + strActnMsk + "\" UsrGrpTyp=\"" + usrGrpID + "\" />");
		}
		sbPermission.append("</List>");
		
		String permissions = sbPermission.toString();
//		logger.info("changePermissions - NEW permissionsList " + permissions);		
		fpcController.changePermissions(cmpntID,permissions);
		
		logger.info("changePermissions - END");		
	}

		


}
