package com.ibm.gre.notifications;

import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.util.Enumeration;

import javax.jms.MapMessage;
import javax.jms.TextMessage;

import com.ibm.gre.engine.types.PermissionsChangedEventTypes;
import com.ibm.gre.engine.util.BatchResultStatusTypes;
import com.ibm.gre.engine.util.MessageKeys;
import com.ibm.gre.engine.util.MessageType;

/**
 * Bean implementation class for Enterprise Bean: IRMNotification
 */
public class IRMNotificationBean
	implements javax.ejb.MessageDrivenBean, javax.jms.MessageListener {
	private javax.ejb.MessageDrivenContext fMessageDrivenCtx;
	/**
	 * getMessageDrivenContext
	 */
	public javax.ejb.MessageDrivenContext getMessageDrivenContext() {
		return fMessageDrivenCtx;
	}
	/**
	 * setMessageDrivenContext
	 */
	public void setMessageDrivenContext(javax.ejb.MessageDrivenContext ctx) {
		fMessageDrivenCtx = ctx;
	}
	/**
	 * ejbCreate
	 */
	public void ejbCreate() {
	}
	/**
	 * onMessage
	 */
	public void onMessage(javax.jms.Message msg) {
		
		String strAppend = "";
		String permChangedEventType = null;
		Enumeration enumMapNames = null;
		try{

			if(msg.getStringProperty(MessageKeys.EVENT_TYPE).equals(MessageType.SECURITY_NOTIFICATION_EVENT))
			{
				//Note for this event type a MapMessage object is used
				MapMessage mapMess = (MapMessage)msg;
				permChangedEventType = mapMess.getString(MessageKeys.PERMISSIONS_CHANGED_EVENT_TYPE);
				
				//Every permission changed event type has a different collection of mapped values
				enumMapNames = mapMess.getMapNames();
				if(permChangedEventType != null)
				{
					strAppend += "\nPermissionsChangedEventType: " + permChangedEventType;
					while(enumMapNames.hasMoreElements())
					{
						String strMapName = enumMapNames.nextElement().toString();
						strAppend += " Mapped Name: " + strMapName + " Value = " + mapMess.getString(strMapName) + " \n";
					}
				}
				
				//These are all the different type of Permission Change Event Types
				if(permChangedEventType.equals(PermissionsChangedEventTypes.ACTIVATE_GROUP))
				{
					//Called by updating group
					//Mapped Names:
					//XT_GROUP_ID
				}
				else if(permChangedEventType.equals(PermissionsChangedEventTypes.ACTIVATE_USER))
				{
					//Called by updating user
					//Mapped Names:
					//XT_USER_ID
				}
				else if(permChangedEventType.equals(PermissionsChangedEventTypes.CHANGED_CUSTODIAN))
				{
					//Called by addFilePlanComponent (part of proccessing the inherited fields)
					//Mapped Names:
					//XT_REC_ID
					
					//Called by updateFilePlanComponent
					//Mapped Names:
					//XT_REC_ID
				}
				else if(permChangedEventType.equals(PermissionsChangedEventTypes.CHANGED_ENGINE_CONFIG))
				{
					//Whenever a value is set for a System Configuration Value (type=0)
					//Mapped Names:
					//Configuration Key that is being set
				}
				else if(permChangedEventType.equals(PermissionsChangedEventTypes.CHANGED_SECURITY_DESCRIPTORS))
				{
					//Called by addUser, updateUser
					//Mapped Names:
					//XT_USER_ID
					
					//Callled by updateGroup
					//Mapped Names
					//XT_GROUP_ID
					
					//Called by addFilePlanComponent (part of proccessing the inherited fields)
					//Mapped Names:
					//XT_REC_ID
					
					//Called by updateFilePlanComponent
					//Mapped Names:
					//XT_REC_ID
					
				}
				else if(permChangedEventType.equals(PermissionsChangedEventTypes.CHANGED_SECURITY_LEVEL))
				{
					//Called by addFilePlanComponent (part of proccessing the inherited fields)
					//Mapped Names:
					//XT_REC_ID
					
					//Called by updateFilePlanComponent
					//Mapped Names:
					//XT_REC_ID
					
					//Called by updateUser
					//Mapped Names:
					//XT_USER_ID
					
				}
				else if(permChangedEventType.equals(PermissionsChangedEventTypes.DEACTIVATE_GROUP))
				{
					//Called by updating group
					//Mapped Names:
					//XT_GROUP_ID
				}
				else if(permChangedEventType.equals(PermissionsChangedEventTypes.DEACTIVATE_USER))
				{
					//Called by updating user
					//Mapped Names:
					//XT_USER_ID
				}
				else if(permChangedEventType.equals(PermissionsChangedEventTypes.REMOVE_GROUP))
				{
					//Called by deleteGroup
					//Mapped Names:
					//XT_GROUP_ID
				}
				else if(permChangedEventType.equals(PermissionsChangedEventTypes.REMOVE_USER))
				{
					//Callled by deleteUser
					//Mapped Names:
					//XT_USER_ID
				}
				else if(permChangedEventType.equals(PermissionsChangedEventTypes.UPDATE_FILE_PLAN_COMPONENT_PERMISSIONS))
				{
					//Called by: changePermissionsOnTransition, changePermissions, deletePermissions, inheritFromParent
					//resetDescendents, setPermissions
					//Mapped Names:
					//FPC_ID
				}
				else if(permChangedEventType.equals(PermissionsChangedEventTypes.UPDATE_ROOT_PERMISSIONS))
				{
					//Called By: changePermissions, deletePermissions, inheritFromParent, resetDescendents
					//setPermissions
					//Mapped Names:
				}
			}
			else if(msg.getStringProperty(MessageKeys.EVENT_TYPE).equals(MessageType.PERMISSIONS_SYNCHRONIZATION_EVENT))
			{
				//Note for this event type a TextMessage object is used
				TextMessage textMess = (TextMessage)msg;
				strAppend = "\nXML: " + textMess.getText(); //XN\ML formatted task info 
				
				long nStatus = textMess.getLongProperty("Status");
				
				switch((int)nStatus)
				{
					case BatchResultStatusTypes.STARTED:
						//batch process has started
					break;
					case BatchResultStatusTypes.CANCELED:
						//batch process was cancelled
					break;
					case BatchResultStatusTypes.COMPLETED:
						//batch process has completed
					break;
					case BatchResultStatusTypes.FAILED:
						//batch process has failed
					break;
				}
			}

			String strMsg = MessageKeys.EVENT_TYPE + ": " + msg.getStringProperty(MessageKeys.EVENT_TYPE) +
							" " + strAppend + " END       ";
			
			System.out.println(strMsg);
			System.out.println(msg.toString());

			SaveToFile(strMsg);

		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}

	private void SaveToFile(String str){
		try {
			OutputStreamWriter osw = new OutputStreamWriter(new FileOutputStream("C:\\IRMNotifications.txt", true), "UTF-8");
			osw.write(str);
			osw.flush();
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
}
