/*
 * Created on Jan 31, 2005
 */
package com.ibm.gre.samples.collectionhelpers;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import com.ibm.gre.engine.util.AttributeControlMask;
import com.ibm.gre.engine.util.AttributeDataTypes;
import com.ibm.gre.engine.util.ProfileControlType;
import com.ibm.gre.engine.util.XMLUtil;

/**
 * The IRMField class encapsulates a Field element from
 * a Records manager Complex Fields Collection returned by one of
 * the TemplateControllerEJB methods.
 * The class contains methods for getting attribute values,
 * getting and setting Field values and other utility functions.
 */
public class IRMField {


	/**
	 * Constructor
	 * @param field An XML Field element from a Records Manager Fields Collection
	 */
	public IRMField(Element field)
	{
		m_field = field;
	}

	/**
	 * Returns the constant value specifying the type of control to display.
	 * @return int
	 * @throws Exception
	 */
	public int getControlType() throws Exception
	{
		String def = Integer.toString(ProfileControlType.SINGLELINE);
		return Integer.parseInt(getVal("ControlType",def));
	}

	/**
	 * Returns the FlPlnCmpntAttrDefColNm for this field.
	 * @return File Plan Component Attribute Column Name
	 * @throws Exception
	 */
	public String ToString() throws Exception
	{
		return getFlPlnCmpntAttrDefColNm();
	}

	/**
	 * File Plan Component Attribute Column Name
	 * @return String
	 * @throws Exception
	 */
	public String getFlPlnCmpntAttrDefColNm() throws Exception
	{
		if(m_strFlPlnCmpntAttrDefColNm == null)
		{
			m_strFlPlnCmpntAttrDefColNm = XMLUtil.getValue(m_field.getAttributes().getNamedItem("FlPlnCmpntAttrDefColNm"));
		}
		return m_strFlPlnCmpntAttrDefColNm;
	}

	/**
	 * File Plan Component Definition ID
	 * @return int
	 * @throws Exception
	 */
	public int getFlPlnCmpntDefID()throws Exception
	{
		return Integer.parseInt(getVal("FlPlnCmpntDefID","0"));
	}

	/**
	 * File Plan Component Attribute Definition Data Type
	 * @return int
	 * @throws Exception
	 */
	public int getFlPlnCmpntAttrDefDatTyp() throws Exception
	{
		String def = "" + AttributeDataTypes.STRING;
		return Integer.parseInt(getVal("FlPlnCmpntAttrDefDatTyp",def));
	}

	/**
	 * Attribute Control Mask
	 * @return int
	 * @throws Exception
	 */
	public int getAttrCntrlMsk() throws Exception
	{
		return Integer.parseInt(getVal("AttrCntrlMsk","0"));
	}

	/**
	 * Caption
	 * @return String
	 * @throws Exception
	 */
	public String getCaption() throws Exception
	{
		if(m_strCaption == null)
		{
			m_strCaption = getVal("Caption","");
		}
		return m_strCaption;
	}

	/**
	 * Field Level Access
	 * @return boolean
	 * @throws Exception
	 */
	public boolean getFldLvlAccess() throws Exception
	{
		return (getVal("FldLvlAccess","1") != "0");
	}

	/**
	 * Mandatory flag.
	 * @return boolean
	 * @throws Exception
	 */
	public boolean getMandatory() throws Exception
	{
		return (getVal("Mandatory","0") != "0");
	}

	/**
	 * Multi valued flag.
	 * @return boolean
	 * @throws Exception
	 */
	public boolean getMultiValuedFlg() throws Exception
	{
		return (getVal("MultiValuedFlg","0") != "0");
	}

	/**
	 * Read Only flag.
	 * @return boolean
	 * @throws Exception
	 */
	public boolean getReadOnly() throws Exception
	{
		return (getVal("ReadOnly","0") != "0");
	}

	/**
	 * Returns true if this field contains a personal Pick List.
	 * @return boolean
	 * @throws Exception
	 */
	public boolean getIsPersonalPickList() throws Exception
	{
		return (getVal("IsPersonalPickList","0") != "0");
	}

	/**
	 * Data Type Maximum Length.
	 * @return int
	 * @throws Exception
	 */
	public int getDatTypMaxLngth() throws Exception
	{
		return Integer.parseInt(getVal("DatTypMaxLngth","0"));
	}

	/**
	 * Pick List ID.
	 * @return int
	 * @throws Exception
	 */
	public int getPckLstID() throws Exception
	{
		return Integer.parseInt(getVal("PckLstID","0"));
	}

	/**
	 *  Returns the XML Field element with all attributes removed
	 *  except the FlPlnCmpntAttrDefColNm attribute.
	 * @return The XML Field element.
	 */
	public Element getFieldForAdd()
	{
		Element field = (Element)m_field.cloneNode(true);
		//Remove Extraneous Attributes
		NamedNodeMap attrs = field.getAttributes();
		for(int i = attrs.getLength() - 1; i > -1; i--)
		{
			if(attrs.item(i).getNodeName() != "FlPlnCmpntAttrDefColNm")
			{
				field.removeAttribute(attrs.item(i).getNodeName());
			}
		}
		return field;
	}

	/**
	 * Gets the value for the field.
	 * @return String
	 * @throws Exception
	 */
	public String getValue() throws Exception 
	{
		//If the field is a Virtual Field return inner XML
		if(AttributeControlMask.isVirtual(getAttrCntrlMsk()))
		{
			return XMLUtil.getXML(m_field);
		}
		if(getMultiValuedFlg())
		{
			return XMLUtil.getXML(m_field);
		}
		return XMLUtil.getValue(m_field);
	}

	/**
	 * Sets the value for the field.
	 * @param val
	 * @throws Exception
	 */
	public void setValue(String val) throws Exception
	{
		if(AttributeControlMask.isVirtual(getAttrCntrlMsk()) || getMultiValuedFlg())
		{
			Document doc = XMLUtil.getDocument(val);
			Element root = doc.getDocumentElement();
			m_field.replaceChild(root,m_field.getFirstChild());
		}
		XMLUtil.setValue(m_field,val);
	}

	/**
	 * Sets if the field should be visible on a form.
	 * @return boolean
	 */
	public boolean getIsVisible()
	{
		return m_bVisible;
	}
	
	/**
	 * Gets if the field should be visible on a form.
	 * @param val
	 */
	public void	setIsVisible(boolean val)
	{
		m_bVisible = val;
	}

	/**
	 * Gets if the field should be used.
	 * @return boolean
	 */
	public boolean getIsActive()
	{
		return m_bIsActive;
	}
	
	/**
	 * Sets if the field should be used.
	 * @param val
	 */
	public void	setIsActive(boolean val)
	{
		m_bIsActive = val;
	}

	/**
	 *  Gets the value for a specified attribute
	 * @param attrName The name of the attribute to return the value for.
	 * @param defaultVal The default value to use if the attribute does not exist or has no value.
	 * @return String
	 * @throws Exception
	 */
	private String getVal(String attrName, String defaultVal) throws Exception
	{
		Node node = m_field.getAttributes().getNamedItem(attrName);
		if(node != null)
		{
			String temp = XMLUtil.getValue(node);
			if(temp.length() > 0)
			{
				defaultVal = temp;
			}
		}
		return defaultVal;
	}

	private Element m_field;
	//These fields are usually used many times so save them
	private String m_strCaption = null;
	private String m_strFlPlnCmpntAttrDefColNm = null;
	private boolean m_bVisible = true;
	private boolean m_bIsActive = true;

}
